--[[ 

	Hardcoded Addon Profiles
	Martin Karer / Sezz, 2010

	Inspired by Ampere by Tekkub Stoutwrithe + addonswitcher by Drauer

	Changelog:
	2010-11-11:
		[*] Equipment Manager fixed (there's no option to disable that stuff anymore?)

	2010-09-18:
		[:] Cataclysm Update

	2010-07-03:
		[+] Equipment Manager
			Helm is hidden by default and gets displayed only if your set name contains "fishing" or +H!

	2010-06-30:
		[+] Recount toggle added if recount is enabled

	2010-02-06:
		[+] Initial Release

--]]

local addonName = "AddonProfiles";
local addon = CreateFrame("Button", addonName);

local addonProfiles = {
	[1] = {
		["Name"] = "Default",
		["DisabledAddons"] = {},
		["EnabledAddons"] = {},
		},
	[2] = {
		["Name"] = "Raid",
		["DisabledAddons"] = {},
		["EnabledAddons"] = {
			"DXE.*",
			"KLE.*",
			"Grid.*",
			"Recount",
			}
		},
	[3] = {
		["Name"] = "PVP",
		["DisabledAddons"] = {},
		["EnabledAddons"] = {
			"Gladius",
			"Grid.*",
			}
		},
	[4] = {
		["Name"] = "Dungeons",
		["DisabledAddons"] = {},
		["EnabledAddons"] = {
			"Recount",
			}
		},
	}

local addonToggle = {
	-- Enable/Disable rarely used Addons
	[1] = {
		["Name"] = "AtlasLoot",
		["Pattern"] = "AtlasLoot.*",
		},
	[2] = {
		["Name"] = "NPC Scan",
		["Pattern"] = "_NPCScan.*",
		},
	[3] = {
		["Name"] = "FishingBuddy",
		["Pattern"] = "FishingBuddy.*",
		},
	[4] = {
		["Name"] = "OverAchiever",
		["Pattern"] = "Overachiever.*",
		},
	[5] = {
		["Name"] = "GearScoreLite",
		["Pattern"] = "GearScoreLite",
		},
	[6] = {
		["Name"] = "Auctionator",
		["Pattern"] = "Auctionator",
		},
	[7] = {
		["Name"] = "Outfitter",
		["Pattern"] = "Outfitter",
		},
	[8] = {
		["Name"] = "Recount",
		["Pattern"] = "Recount.*",
		},
	[9] = {
		["Name"] = "GatherMate",
		["Pattern"] = "GatherMate.*",
		},
	[10] = {
		["Name"] = "AceConsole",
		["Pattern"] = "AceConsole.*",
		},
	[11] = {
		["Name"] = "Archy",
		["Pattern"] = "Archy.*",
		},
	[12] = {
		["Name"] = "ReforgeLite",
		["Pattern"] = "ReforgeLite",
		},
	[13] = {
		["Name"] = "TinyDPS",
		["Pattern"] = "TinyDPS",
		},
	}

local ToggleProfile = function(profileIndex)
	for k, v in pairs(addonProfiles) do
		if (k == profileIndex) then
			v.Checked = v.Checked == nil and true or nil;
			addon.Profile = v.Checked and k or nil;
		else
			v.Checked = nil;
		end
	end
end

local ToggleAddon = function(addonIndex)
	local addonData = addonToggle[addonIndex];
	if (addonData ~= nil) then
		addonData.Enabled = not addonData.Enabled;
	end
end

table.find = function(table, value)
	for k, v in ipairs(table) do
		if (v == value) then
			return k;
		end
	end
	
	return false;
end

local ApplyConfiguration = function()
	local EnabledAddons = {}
	local DisabledAddons = {}
	
	-- Profile Addon List
	-- TODO: Shorten this, but I'm a little bit tired (6:50 AM)
	if (addon.Profile) then
		local profileData;
		local defaultProfile;

		-- Disable other profiles enabled addons
		for i = 1, #addonProfiles do
			if (i ~= addon.Profile and addonProfiles[i].Name ~= "Default") then
				for _, v in pairs(addonProfiles[i].EnabledAddons) do
					if (not table.find(DisabledAddons, v) and not table.find(addonProfiles[addon.Profile].EnabledAddons, v)) then
						table.insert(DisabledAddons, v);
					end
				end
			elseif (not defaultProfile and addonProfiles[i].Name == "Default") then
				defaultProfile = i;
			end
		end
		
		-- Disable default addons (if not enabled in chosen profile)
		for _, v in pairs(addonProfiles[defaultProfile].DisabledAddons) do
			if (not table.find(DisabledAddons, v) and (addon.Profile == defaultProfile or not table.find(addonProfiles[addon.Profile].EnabledAddons, v))) then
				table.insert(DisabledAddons, v);
			end
		end

		-- Enable default addons (if not disabled and chosen profile)
		for _, v in pairs(addonProfiles[defaultProfile].EnabledAddons) do
			if (not table.find(EnabledAddons, v) and (addon.Profile == defaultProfile or not table.find(addonProfiles[addon.Profile].DisabledAddons, v))) then
				table.insert(EnabledAddons, v);
			end
		end
		
		-- Enable chosen profile addons
		if (addon.Profile ~= defaultProfile) then
			for _, v in pairs(addonProfiles[addon.Profile].EnabledAddons) do
				if (not table.find(EnabledAddons, v)) then
					table.insert(EnabledAddons, v);
				end
			end
		end
	end
	
	-- Custom Addon List
	local i;
	for k, v in pairs(addonToggle) do
		if (v.Enabled ~= nil) then
			if (v.Enabled) then
				-- Enable Addon
				i = table.find(DisabledAddons, v.Pattern);
				if (i) then table.remove(DisabledAddons, i); end
				if (not table.find(EnabledAddons, v.Pattern)) then
					table.insert(EnabledAddons, v.Pattern);
				end
			else
				-- Disable Addon
				i = table.find(EnabledAddons, v.Pattern);
				if (i) then table.remove(EnabledAddons, i); end
				if (not table.find(DisabledAddons, v.Pattern)) then
					table.insert(DisabledAddons, v.Pattern);
				end
			end
		end
	end
	
	-- Enable/Disable Addons
	for i = 1, GetNumAddOns(), 1 do
		local name = GetAddOnInfo(i)

		for _, v in pairs(DisabledAddons) do
			if (string.find(name, v)) then
				DisableAddOn(name);
			end
		end

		for _, v in pairs(EnabledAddons) do
			if (string.find(name, v)) then
				EnableAddOn(name);
			end
		end
	end

	ReloadUI();
end

-- Context Menu
local subMenuItemData = {};
local CreateSubMenuItem = function(text, root, func)
	subMenuItemData.hasArrow = true;
	subMenuItemData.notCheckable = true;
	subMenuItemData.text = text;
	subMenuItemData.value = {
		["Parent"] = text,
		["Root"] = root,
		};
	subMenuItemData.func = func;
	return subMenuItemData;
end

local CreateMenuItem = function(text, value, checkable, checked, func)
	-- checkable: 0 = disabled, 1 = checkbox, 2 = radio 
	local menuItem = {}
	menuItem.text = text;
	menuItem.value = value;
	if (checkable > 0) then
		if (checkable ~= 2) then menuItem.keepShownOnClick = true; menuItem.isNotRadio = true; end;
		if (checkable ~= 3) then menuItem.checked = type(checked) == "function" and checked() or checked; end;
	else
		menuItem.notCheckable = true;
	end
	menuItem.func = func;
	return menuItem;
end

local InitializeContextMenu = function(self, level)
	level = level or 1;

	if (level == 1) then
		-- Sub Menus
		UIDropDownMenu_AddButton(CreateSubMenuItem("Profiles"));
		UIDropDownMenu_AddButton(CreateSubMenuItem("Addons"));
		UIDropDownMenu_AddButton(CreateSubMenuItem("Equipment"));
		-- Recount
		if (IsAddOnLoaded("Recount")) then
			UIDropDownMenu_AddButton(CreateMenuItem("Toggle Recount", nil, 0, false, Recount.consoleOptions.args.toggle.func), level);
		end
		-- Reload UI
		UIDropDownMenu_AddButton(CreateMenuItem("Apply", nil, 0, false, ApplyConfiguration), level);
	elseif (level == 2) then
		if (UIDROPDOWNMENU_MENU_VALUE["Parent"] == "Profiles") then
			-- Profiles
			for k, v in ipairs(addonProfiles) do
				UIDropDownMenu_AddButton(CreateMenuItem(v.Name, k, 2, v.Checked ~= nil, function() ToggleProfile(k); end), level);
			end
		elseif (UIDROPDOWNMENU_MENU_VALUE["Parent"] == "Addons") then
			-- Addons
			for k, v in ipairs(addonToggle) do
				if (v.Enabled ~= nil) then
					UIDropDownMenu_AddButton(CreateMenuItem(v.Name, k, 1, v.Enabled, function() ToggleAddon(k); end), level);
				end
			end
		elseif (UIDROPDOWNMENU_MENU_VALUE["Parent"] == "Equipment") then
			-- Equipment Manager
--			if (GetCVar("equipmentManager") == "1") then
				for i=1, GetNumEquipmentSets() do
					local eqName, _, _ = GetEquipmentSetInfo(i);
					UIDropDownMenu_AddButton(CreateMenuItem(eqName, nil, 0, false, function() UseEquipmentSet(eqName); end), level);
--					UIDropDownMenu_AddButton(CreateSubMenuItem(eqName, UIDROPDOWNMENU_MENU_VALUE["Parent"], function() UseEquipmentSet(eqName); end), level);
				end
--			end
		end
--	elseif (level == 3) then
--		if (UIDROPDOWNMENU_MENU_VALUE["Root"] == "Equipment") then
--			-- Profile Manager
--			UIDropDownMenu_AddButton(CreateMenuItem("Equip", nil, 0, false, function() print(UIDROPDOWNMENU_MENU_VALUE["Parent"]) end), level);
--			UIDropDownMenu_AddButton(CreateMenuItem("Update", nil, 0, false, function() print(UIDROPDOWNMENU_MENU_VALUE["Parent"]) end), level);
--		end
	end
end

local tableSort = function(a, b)
	return string.lower(a.Name) < string.lower(b.Name);
end

addon.EQUIPMENT_SWAP_FINISHED = function(self, event, changed, name)
	if (changed) then
		if (string.find(strlower(name), "fishing") or string.find(strlower(name), "+h")) then
			-- show head
			ShowHelm(true);
		else
			-- hide head
			ShowHelm(false);
		end
	else
		-- Failed equipping set
		sCore:Print("|cffff0000Failed equipping set "..name.."!|r");
	end
end

addon.PLAYER_LOGIN = function(self, event, ...)
	-- Sort Addons
	table.sort(addonToggle, tableSort);
	table.sort(addonProfiles, tableSort);

	-- Get Addons States
	for i = 1, GetNumAddOns(), 1 do
		local name, _, _, enabled = GetAddOnInfo(i)
		for k, v in pairs(addonToggle) do
			if (string.find(name, v.Pattern)) then
				if (v.Enabled == nil) then v.Enabled = true; end;
				v.Enabled = v.Enabled and (not not enabled);
			end
		end
	end
		
	-- Create Button
	self:SetScale(0.7);
	self:SetWidth(MiniMapTracking:GetWidth())
	self:SetHeight(MiniMapTracking:GetHeight())
	self:SetPoint("RIGHT", UIParent, "RIGHT", -40, -40);
	self:SetScript("OnMouseUp", function() ToggleDropDownMenu(1, nil, self.Menu, self, 0, 0); end);
	self:SetScript("OnEnter", function() GameTooltip:SetOwner(self, "ANCHOR_LEFT"); GameTooltip:SetText("Addons"); end)
	self:SetScript("OnLeave", function() GameTooltip:Hide(); end)
	self:SetParent(Minimap)
	self:ClearAllPoints()
	self:SetPoint("BOTTOMLEFT", 160, -70)
	sCore:ApplyBackdropBorder(self);
	
	local icon = self:CreateTexture(nil, "OVERLAY");
	icon:SetTexture("Interface\\Icons\\Trade_Engineering");
	icon:SetTexCoord(0.08, 0.92, 0.08, 0.92); 
	icon:SetPoint("TOPLEFT", self, "TOPLEFT", 2, -2)
	icon:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", -2, 2)
	
	-- Create Context Menu
	local contextMenu = CreateFrame("Frame", addonName.."Menu", self, "UIDropDownMenuTemplate");
	UIDropDownMenu_Initialize(contextMenu, InitializeContextMenu, "MENU");
	self.Menu = contextMenu
	
	-- Done
	self.DisableEvent(self, event);
end

addon.DisableEvent = function(self, event, ...)
	self:UnregisterEvent(event);
	self[event] = nil;
end

addon:RegisterEvent("PLAYER_LOGIN");
addon:RegisterEvent("EQUIPMENT_SWAP_FINISHED");
addon:SetScript("OnEvent",function(self, event, ...) self[event](self, event, ...); end);

