-- Martin Karer / Sezz, 2008-2011
-- BurroBag: Automatically opens and closes your Bags.
-- 2008-??-??: Initial release...
-- 2011-05-04: 4.1 Fixes
-- 2011-08-01: Fixed a bug, when all bank bag slots are empty

local BurroBag = CreateFrame("Frame")
local _lstEvents = {
	"MERCHANT_SHOW",
	"MERCHANT_CLOSED",
	"TRADE_SHOW",
	"TRADE_CLOSED",
	"MAIL_SHOW",
	"MAIL_CLOSED",
	"BANKFRAME_OPENED",
	"BANKFRAME_CLOSED",
	"AUCTION_HOUSE_SHOW",
	"AUCTION_HOUSE_CLOSED",
	"PLAYER_ENTERING_WORLD",
	"GUILDBANKFRAME_OPENED",
	"GUILDBANKFRAME_CLOSED"
	}

local BankHasBags = function()
	for i = NUM_BAG_FRAMES + 1, NUM_CONTAINER_FRAMES, 1 do
		if ( GetContainerNumSlots(i) > 0 ) then     
			return true;
		end
	end
	
	return false;
end

function ReallyOpenAllBags()
	CloseAllBags();
	OpenAllBags();
	if (BankFrame and BankFrame:IsVisible() and BankHasBags()) then ToggleAllBags(); end
end

function ReallyToggleAllBags()
	if (not IsBagOpen(0)) then
		ReallyOpenAllBags();
	else
		CloseAllBags();
	end
end

local OnEvent = function(self, event)
	CloseAllBags();
	if (event:match("_SHOW$") or event:match("_OPENED$")) then
		ReallyOpenAllBags();
	elseif (event == "PLAYER_ENTERING_WORLD") then
		SetBinding("B", "REALLYTOGGLEALLBAGS");
	end
end

for _, sEvent in pairs(_lstEvents) do
	BurroBag:RegisterEvent(sEvent);
end

BurroBag:SetScript("OnEvent", OnEvent)
